@@ -34,6 +34,7 @@ import pathlib
 import shlex
 import subprocess
 import sys
+import urllib.error
 import urllib.parse
 import urllib.request
 
@@ -41,18 +42,20 @@ import urllib.request
 class ArchiveCommand:
     """Object for storing information needed to extract archives."""
 
-    def __init__(self, decomp_cmd="", pipe_cmd="", uses_stdin=False, uses_stdout=False):
+    def __init__(
+        self, extract_cmd="", pipe_cmd="", uses_stdin=False, uses_stdout=False
+    ):
         """Set attributes for decompression and piping.
 
-        :param decomp_cmd: command string to decompress archive
-        :param pipe_cmd: command string to pipe
-        :param uses_stdin: boolean value if decomp_cmd uses stdin
-        :param uses_stdout: boolean value decomp_cmd uses stdout
+        @param extract_cmd: command string to decompress archive
+        @param pipe_cmd: command string to pipe
+        @param uses_stdin: boolean value if extract_cmd uses stdin
+        @param uses_stdout: boolean value extract_cmd uses stdout
 
-        :returns: None
+        @returns: None
         """
 
-        self.decomp_cmd = decomp_cmd
+        self.extract_cmd = extract_cmd
         self.pipe_cmd = pipe_cmd
         self.uses_stdin = uses_stdin
         self.uses_stdout = uses_stdout
@@ -60,10 +63,10 @@ class ArchiveCommand:
     def __repr__(self):
         """Representation of the object.
 
-        :returns: string representing ArchiveCommand
+        @returns: string representing ArchiveCommand
         """
 
-        msg = f"[ decomp_cmd = {self.decomp_cmd!r} "
+        msg = f"[ extract_cmd = {self.extract_cmd!r} "
         msg += f"pipe_cmd = {self.pipe_cmd!r} "
         msg += f"uses_stdin = {self.uses_stdin!r} "
         msg += f"uses_stdout = {self.uses_stdout!r} ] "
@@ -73,9 +76,9 @@ class ArchiveCommand:
 def command_exists(cmd):
     """Test for an external command's existence.
 
-    :param cmd: external command to check for existence
+    @param cmd: external command to check for existence
 
-    :returns: boolean value True if cmd exists, False otherwise
+    @returns: boolean value True if cmd exists, False otherwise
     """
 
     try:
@@ -90,50 +93,45 @@ def command_exists(cmd):
 def glob_multiple_extensions(extensions):
     """Iterate over a sequence of extensions to glob.
 
-    :param extensions: a sequence of extensions to iterate over
+    @param extensions: a sequence of extensions to iterate over
 
-    :returns: a list of positive matches globbed
+    @returns: a list of positive glob matches
     """
 
-    files_globbed = []
+    glob_files = []
 
     for ext in extensions:
-        files_globbed.extend(glob.glob(ext))
+        glob_files.extend(glob.glob(ext))
 
-    return files_globbed
+    return glob_files
 
 
-def simple_extract(archive, archive_cmd, noclobber=False):
+def simple_extract(archive, archive_cmd, no_clobber=False):
     """Extract an archive using external tools.
 
-    :param archive: the archive to be extracted
-    :param archive_cmd: a completed ArchiveCommand object
-    :param noclobber: boolean option not to overwrite existing files (doesn't work with pipes)
+    @param archive: the archive to be extracted
+    @param archive_cmd: a completed ArchiveCommand object
+    @param no_clobber: boolean option not to overwrite existing files
 
-    :returns: None
+    @returns: None
     """
 
     uses_stdin = archive_cmd.uses_stdin
     uses_stdout = archive_cmd.uses_stdout
 
     if uses_stdin or uses_stdout:
-        decomp_cmd = shlex.split(archive_cmd.decomp_cmd)
+        extract_cmd = shlex.split(archive_cmd.extract_cmd)
     else:
-        decomp_cmd = shlex.split(archive_cmd.decomp_cmd + " " + archive)
+        extract_cmd = shlex.split(archive_cmd.extract_cmd + " " + archive)
 
     pipe_cmd = shlex.split(archive_cmd.pipe_cmd)
 
     # Valid extension suffixes
     valid_suffixes = (
-        ".tar.bz2",
         ".tbz2",
         ".tbz",
-        ".tar.gz",
         ".tgz",
-        ".tar.xz",
         ".txz",
-        ".tar.lzma",
-        ".tar.zst",
         ".tar",
         ".rar",
         ".lzh",
@@ -159,7 +157,7 @@ def simple_extract(archive, archive_cmd, noclobber=False):
 
     print(f"Target: {target}")
 
-    if os.path.exists(target) and noclobber:
+    if os.path.exists(target) and no_clobber:
         print(f"Target: {target} already exists not overwriting...")
         return
 
@@ -167,42 +165,67 @@ def simple_extract(archive, archive_cmd, noclobber=False):
     with open(archive) as fin:
         if not pipe_cmd:
             if uses_stdin and not uses_stdout:
-                # use a context manager so each subprocess is waited on
-                with subprocess.Popen(decomp_cmd, stdin=fin) as sproc:
-                    sproc.communicate()
+                try:
+                    subprocess.run(extract_cmd, stdin=fin, check=True)
+                except subprocess.CalledProcessError as e:
+                    print(f"Error: Return Code = {e.returncode} {e.output or ''}")
             elif uses_stdin and uses_stdout:
-                with open(target, "w+") as fout:
-                    with subprocess.Popen(decomp_cmd, stdin=fin, stdout=fout) as sproc:
-                        sproc.communicate()
+                with open(target, "w+") as outfile:
+                    try:
+                        subprocess.run(
+                            extract_cmd, stdin=fin, stdout=outfile, check=True
+                        )
+                    except subprocess.CalledProcessError as e:
+                        print(f"Error: Return Code = {e.returncode} {e.output or ''}")
+                        os.remove(target)
             else:
-                with subprocess.Popen(decomp_cmd) as _:
-                    pass
+                try:
+                    subprocess.run(extract_cmd, check=True)
+                except subprocess.CalledProcessError as e:
+                    print(f"Error: Return Code = {e.returncode} {e.output or ''}")
         else:
-            with subprocess.Popen(decomp_cmd, stdin=fin, stdout=subprocess.PIPE) as cmd:
-                subprocess.run(pipe_cmd, stdin=cmd.stdout, check=True)
+            with subprocess.Popen(
+                extract_cmd, stdin=fin, stdout=subprocess.PIPE
+            ) as cmd:
+                try:
+                    subprocess.run(pipe_cmd, stdin=cmd.stdout, check=True)
+                except subprocess.CalledProcessError as e:
+                    print(f"Error: Return Code = {e.returncode} {e.output or ''}")
 
 
 def should_fetch_url(archive_url, local_archive):
     """Check if an archive should be fetched by comparing remote and local file sizes.
 
-    :param archive_url: url of archive to be downloaded
-    :param local_archive: possible local archive that may already have been downloaded
+    @param archive_url: url of archive to be downloaded
+    @param local_archive: possible local archive that may already have been downloaded
 
-    :returns: boolean True if archive should downloaded, False otherwise
+    @returns: boolean True if archive should downloaded, False otherwise
     """
 
-    # return False if local archive does not exist
-    if not os.path.exists(local_archive):
-        return True
+    print(f"Validating archive url: {archive_url}")
 
     # get content-size of remote archive
     req = urllib.request.Request(archive_url, method="HEAD")
-    with urllib.request.urlopen(req) as f:
-        remote_size = f.headers["content-length"]
+    try:
+        with urllib.request.urlopen(req) as f:
+            remote_size = f.headers["content-length"]
+    except urllib.error.HTTPError as e:
+        print("Error: The server couldn't fulfil the request")
+        print(f"Error Code: {e.code}")
+        return False
+    except urllib.error.URLError as e:
+        print("Error: failed to reach the server")
+        print(f"Reason: {e.reason}")
+        return False
+
+    # archive should be fetched if a local copy does not exist
+    if not os.path.exists(local_archive):
+        return True
 
     # get size of local archive
     local_size = os.path.getsize(local_archive)
 
+    print(f"Comparing remote and local archives")
     print(f"remote size: {remote_size}, local size: {local_size}")
 
     # compare remote and local sizes, if equal return False
@@ -222,10 +245,10 @@ def should_fetch_url(archive_url, local_archive):
 def fetch_archive(url, silent_download=False):
     """Download an archive for extraction.
 
-    :param url: url of archive to be downloaded
-    :param silent_download: boolean switch to quiet download output
+    @param url: url of archive to be downloaded
+    @param silent_download: boolean switch to quiet download output
 
-    :returns: boolean False if failure, target archive if successful
+    @returns: boolean False if failure, target archive if successful
     """
 
     _, target = os.path.split(url)
@@ -251,13 +274,17 @@ def fetch_archive(url, silent_download=False):
 
     # Check if an archive should be downloaded
     if not should_fetch_url(url, target):
-        return False
+        return target
 
     print(f"Fetching archive {target}")
 
-    with open(target, "w+") as fout:
-        with subprocess.Popen(fetch_cmd, stdin=subprocess.PIPE, stdout=fout) as sproc:
-            sproc.communicate()
+    with open(target, "w+") as outfile:
+        try:
+            subprocess.run(fetch_cmd, stdin=subprocess.PIPE, stdout=outfile, check=True)
+        except subprocess.CalledProcessError as e:
+            print(f"Error: Return Code = {e.returncode} {e.output or ''}")
+            os.remove(target)
+            return False
 
     return target
 
@@ -265,9 +292,9 @@ def fetch_archive(url, silent_download=False):
 def extract_urls(args):
     """Extract urls from a sequence.
 
-    :param args: list of possible valid urls to download
+    @param args: list of possible valid urls to download
 
-    :returns: list of valid urls to download
+    @returns: list of valid urls to download
     """
 
     possibles = [urllib.parse.urlsplit(x) for x in args]
@@ -281,7 +308,7 @@ def main():
     """Main function.
     This is where setup.py defines it's entry-point to create the simple-extract installable.
 
-    :returns: None
+    @returns: None
     """
 
     parser = argparse.ArgumentParser(
@@ -292,13 +319,13 @@ def main():
     parser.add_argument(
         "--version",
         action="version",
-        version="%(prog)s 0.1.8",
+        version="%(prog)s 0.2.1",
     )
     parser.add_argument(
-        "--noclobber",
+        "--no_clobber",
         action="store_true",
         help="Don't overwrite existing files",
-        dest="noclobber",
+        dest="no_clobber",
     )
     parser.add_argument(
         "--silent_download",
@@ -311,8 +338,45 @@ def main():
 
     working_dir = os.getcwd()
     args = [x for x in parsed.ARCHIVES]
-    files_globbed = []
+    glob_files = []
     commands = []
+    command_map = {
+        "*.tar.bz2": ArchiveCommand(extract_cmd="tar -xvjf -", uses_stdin=True),
+        "*.tbz2": ArchiveCommand(extract_cmd="tar -xvjf -", uses_stdin=True),
+        "*.tbz": ArchiveCommand(extract_cmd="tar -xvjf -", uses_stdin=True),
+        "*.tar.gz": ArchiveCommand(extract_cmd="tar -xvzf -", uses_stdin=True),
+        "*.tgz": ArchiveCommand(extract_cmd="tar -xvzf -", uses_stdin=True),
+        "*.tar.xz": ArchiveCommand(extract_cmd="tar -xvJf -", uses_stdin=True),
+        "*.txz": ArchiveCommand(extract_cmd="tar -xvJf -", uses_stdin=True),
+        "*.tar.lzma": ArchiveCommand(extract_cmd="tar -xvJf -", uses_stdin=True),
+        "*.tar.zst": ArchiveCommand(extract_cmd="tar --zstd -xvf -", uses_stdin=True),
+        "*.tar": ArchiveCommand(extract_cmd="tar -xvf -", uses_stdin=True),
+        "*.rar": ArchiveCommand(extract_cmd="unrar x"),
+        "*.lzh": ArchiveCommand(extract_cmd="lha x"),
+        "*.7z": ArchiveCommand(extract_cmd="7z x"),
+        "*.zip": ArchiveCommand(extract_cmd="unzip"),
+        "*.jar": ArchiveCommand(extract_cmd="unzip"),
+        "*.rpm": ArchiveCommand(extract_cmd="rpm2cpio -", pipe_cmd="cpio -idvm"),
+        "*.deb": ArchiveCommand(extract_cmd="ar -x"),
+        "*.bz2": ArchiveCommand(
+            extract_cmd="bzip2 -d -c -", uses_stdin=True, uses_stdout=True
+        ),
+        "*.gz": ArchiveCommand(
+            extract_cmd="gzip -d -c -", uses_stdin=True, uses_stdout=True
+        ),
+        "*.Z": ArchiveCommand(
+            extract_cmd="gzip -d -c -", uses_stdin=True, uses_stdout=True
+        ),
+        "*.xz": ArchiveCommand(
+            extract_cmd="xz -d -c -", uses_stdin=True, uses_stdout=True
+        ),
+        "*.lzma": ArchiveCommand(
+            extract_cmd="xz -d -c -", uses_stdin=True, uses_stdout=True
+        ),
+        "*.zst": ArchiveCommand(
+            extract_cmd="zstd -d -c -", uses_stdin=True, uses_stdout=True
+        ),
+    }
 
     # sanitize inputs
     possibles = [os.path.realpath(x) for x in args]
@@ -327,87 +391,25 @@ def main():
             continue
         archives.append(target)
 
-    print(f"archives: {archives}")
+    if len(archives):
+        print(f"archives: {archives}")
 
-    # create an ArchiveCommand for each archive, pass them to simple_extract
-    for archive in archives:
-        print(f"Examining archive: {archive}")
+        # create an ArchiveCommand for each archive, pass them to simple_extract
+        for archive in archives:
+            print(f"Examining archive: {archive}")
 
-        pn, fn = os.path.split(archive)
-        if pn and archive not in url_archives:
-            os.chdir(pn)
+            pathname, filename = os.path.split(archive)
+            if pathname and archive not in url_archives:
+                os.chdir(pathname)
 
-        if fn in glob_multiple_extensions(("*tar.bz2", "*tbz2", "*tbz")):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="tar -xvjf -", uses_stdin=True)
-            commands.append(cmd)
-        elif fn in glob_multiple_extensions(("*tar.gz", "*tgz")):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="tar -xvzf -", uses_stdin=True)
-            commands.append(cmd)
-        elif fn in glob_multiple_extensions(("*tar.xz", "*txz", "*tar.lzma")):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="tar -xvJf -", uses_stdin=True)
-            commands.append(cmd)
-        elif fn in glob.glob("*tar.zst"):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="tar --zstd -xvf -", uses_stdin=True)
-            commands.append(cmd)
-        elif fn in glob.glob("*tar"):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="tar -xvf -", uses_stdin=True)
-            commands.append(cmd)
-        elif fn in glob.glob("*rar"):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="unrar x")
-            commands.append(cmd)
-        elif fn in glob.glob("*lzh"):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="lha x")
-            commands.append(cmd)
-        elif fn in glob.glob("*7z"):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="7z x")
-            commands.append(cmd)
-        elif fn in glob_multiple_extensions(("*zip", "*jar")):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="unzip")
-            commands.append(cmd)
-        elif fn in glob.glob("*rpm"):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="rpm2cpio -", pipe_cmd="cpio -idvm")
-            commands.append(cmd)
-        elif fn in glob.glob("*deb"):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(decomp_cmd="ar -x")
-            commands.append(cmd)
-        elif fn in glob.glob("*bz2"):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(
-                decomp_cmd="bzip2 -d -c -", uses_stdin=True, uses_stdout=True
-            )
-            commands.append(cmd)
-        elif fn in glob_multiple_extensions(("*gz", "*Z")):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(
-                decomp_cmd="gzip -d -c -", uses_stdin=True, uses_stdout=True
-            )
-            commands.append(cmd)
-        elif fn in glob_multiple_extensions(("*xz", "*lzma")):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(
-                decomp_cmd="xz -d -c -", uses_stdin=True, uses_stdout=True
-            )
-            commands.append(cmd)
-        elif fn in glob.glob("*zst"):
-            files_globbed.append(archive)
-            cmd = ArchiveCommand(
-                decomp_cmd="zstd -d -c -", uses_stdin=True, uses_stdout=True
-            )
-            commands.append(cmd)
+            for extension, command in command_map.items():
+                if filename in glob.glob(extension):
+                    if archive not in glob_files:
+                        glob_files.append(archive)
+                        commands.append(command_map[extension])
 
-    if files_globbed:
-        print(f"Files to extract: {files_globbed}")
+        if glob_files:
+            print(f"Files to extract: {glob_files}")
     else:
         print("Nothing to do.")
         print("Try passing --help as an argument for more information.")
@@ -416,13 +418,13 @@ def main():
     os.chdir(working_dir)
 
     # pass archives and their ArchiveCommand's to simple_extract
-    for archive, archive_cmd in zip(files_globbed, commands):
-        root_cmd = archive_cmd.decomp_cmd.split()[0]
+    for archive, archive_cmd in zip(glob_files, commands):
+        root_cmd = archive_cmd.extract_cmd.split()[0]
         if not command_exists(root_cmd):
             print(f"Error: {root_cmd} does not exist...not extracting {archive}.")
             continue
         print(f"Extracting file {archive}")
-        simple_extract(archive, archive_cmd, noclobber=parsed.noclobber)
+        simple_extract(archive, archive_cmd, no_clobber=parsed.no_clobber)
 
 
 # Program entry point if ran as a normal script, e.g. python simple_extract.py
